package com.jakubu9333.grammarManager;

import com.jakubu9333.grammarManager.model.Grammar;
import com.jakubu9333.grammarManager.model.Rule;
import com.jakubu9333.grammarManager.service.GrammarService;

import org.junit.jupiter.api.Test;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.web.server.ResponseStatusException;

import java.util.ArrayList;
import java.util.List;


/**
 * @author Jakub Uhlarik
 */
@SpringBootTest
public class FileGrammarServiceTest {

    @Autowired
    public GrammarService grammarService;

    @Test
    public void jsgfReadTest() {
        grammarService.deleteAll();
        boolean xml = false;
        Grammar emptyGrammar = grammarService.addGrammarFromFile("", xml);
        assert emptyGrammar.getUri().equals("null");
        assert emptyGrammar.getRules().size() == 0;
        assert emptyGrammar.getRootRule() == null;
        String file1 = """
                #JSGF;
                grammar first;
                rule1 = smile;
                //root;
                rule2 = cry|sis;
                                
                               
                          
                """;
        Grammar firstGrammar = grammarService.addGrammarFromFile(file1, xml);
        assert firstGrammar.getUri().equals("first");
        Rule rule2 = firstGrammar.getRootRule();
        assert rule2.getLeftSymbol().equals("<rule2>");
        assert firstGrammar.getRules().size() == 2;
        assert rule2.getRoot();
        assert rule2.getRightSide().equals("cry|sis");
        Rule rule1 = firstGrammar.getRules().stream().filter(p -> p.getLeftSymbol().equals("<rule1>")).toList().get(0);
        assert rule1.getLeftSymbol().equals("<rule1>");
        assert rule1.getRightSide().equals("smile");
        String file2 = """
                                
                grammar dms;
                ddsd=sfdsfd;
                grammar xfs;
                                
                """;
        boolean error = false;
        try {
            grammarService.addGrammarFromFile(file2, xml);
        } catch (ResponseStatusException e) {
            assert e.getReason().matches("duplicate grammar name declaration;line=[0-9]+");
            error = true;
        }
        assert error;
        String file3 = """
                                
                grammar dms;
                //root;
                ddsd=sfdsfd;
                 //root;
                ddssfsd=sfdsfd;
                                
                """;
        error = false;
        try {
            grammarService.addGrammarFromFile(file3, xml);
        } catch (ResponseStatusException e) {
            error = true;
        }
        assert !error;
    }

    @Test
    public void xmlReadTest() {
        grammarService.deleteAll();
        boolean xml = true;
        boolean er = false;
        try {
            Grammar emptyGrammar = grammarService.addGrammarFromFile("", xml);
        } catch (ResponseStatusException e) {
            er = true;
        }
        assert er;

        String file1 = """
                <?xml version="1.0"?>
                            <grammar version="1.0"
                           xml:base="first"
                             xmlns="http://www.w3.org/2001/06/grammar">
                            <rule id="rule1">
                            <one-of>
                            <item>123
                            </item>
                            <item>smile <ruleref uri= "#rule2"  />haha</item>
                            </one-of>
                            </rule>
                                <rule id="rule2">
                                    <item>cry</item>
                                </rule>
                            </grammar>
                          
                """;
        Grammar firstGrammar = grammarService.addGrammarFromFile(file1, xml);
        assert firstGrammar.getUri().equals("first");
        for (Rule rule : firstGrammar.getRules()) {
            assert predicate1(rule);
        }


        String fileRoot = """
                <?xml version="1.0"?>
                            <grammar version="1.0"
                           xml:base="first"
                           root="rule1"
                             xmlns="http://www.w3.org/2001/06/grammar">
                            <rule id="rule1">
                            <one-of>
                            <item>123
                            </item>
                            <item>smile <ruleref uri= "#rule2"  />haha</item>
                            </one-of>
                            </rule>
                                <rule id="rule2">
                                    <item>cry</item>
                                </rule>
                            </grammar>
                          
                """;
        Grammar rootGr = grammarService.addGrammarFromFile(fileRoot, xml);
        assert rootGr.getRootRule() != null;

    }

    @Test
    public void testErrorMessages() {


        String file = """
                <?xml version="1.0"?>
                            <grammar version="1.0"
                            tag-format="second"
                             xmlns="http://www.w3.org/2001/06/grammar">
                            <rule id="rule1">
                            <one-of>
                            <item>"123"
                            </item>
                            <item>"smile "<ruleref uri= "#rule2"  />haha</item>
                            </one-of>
                     
                                <rule id="rule2">
                                    <item>"cry"</item>
                                </rule>
                            </grammar>
                          
                """;
        String file1 = """
                <?xml version="1.0"?>
                            <grammar version="1.0"
                            tag-format="first"
                             xmlns="http://www.w3.org/2001/06/grammar">
                            </rule>
                            </grammar>
                """;
        String message = "parsing error";
        String message1 = "parsing error";
        List<String> files = new ArrayList<>();
        List<String> messages = new ArrayList<>();
        files.add(file);
        files.add(file1);
        messages.add(message);
        messages.add(message1);
        String file2 = """
                            <rule id="2"> </rule>
                             <?xml version="1.0"?>
                            <grammar version="1.0"
                            tag-format="first"
                             xmlns="http://www.w3.org/2001/06/grammar">
                              </grammar>
                """;
        files.add(file2);
        messages.add("parsing error");

        String file3 = """
                           
                             <?xml version="1.0"?>
                            <grammar version="1.0"
                            tag-format="first"
                             xmlns="http://www.w3.org/2001/06/grammar">
                              <rule id="2"><item> "ddd" </item>
                              </rule>
                              <one-of>
                              </one-of>
                              </grammar>
                """;
        files.add(file3);
        messages.add("parsing error");
        String file4 = """
                           
                             <?xml version="1.0"?>
                            <grammar version="1.0"
                            tag-format="first"
                             xmlns="http://www.w3.org/2001/06/grammar">
                              <rule id="2"> <one-of><item>"bubub"
                              </one-of>
                              </rule>
                             </item>
                              </grammar>
                """;
        files.add(file4);
        messages.add("parsing error");

        String file5 = """
                           
                             <?xml version="1.0"?>
                            <grammar version="1.0"
                            tag-format="first"
                             xmlns="http://www.w3.org/2001/06/grammar">
                              <rule id="2"> <one-of><item>"cry"
                             </item>
                              </rule>
                              </one-of>
                             
                              </grammar>
                """;
        files.add(file5);
        messages.add("parsing error");
        String file6 = """
                           
                             <?xml version="1.0"?>
                            <grammar version="1.0"
                            tag-format="first"
                             xmlns="http://www.w3.org/2001/06/grammar">
                              <rule id="2"><item>"smile"</item>
                              </one-of>
                              </rule>
                             
                              </grammar>
                """;
        files.add(file6);
        messages.add("parsing error");
        String file7 = """
                           
                             <?xml version="1.0"?>
                            <grammar version="1.0"
                            tag-format="first"
                             xmlns="http://www.w3.org/2001/06/grammar">
                              <rule id="2"> <one-of><item>"xd"</item>
                              </one-of>
                              </rule>
                            <item>"outside"</item>
                              </grammar>
                """;
        messages.add("parsing error");
        files.add(file7);
        String file8 = """
                           
                             <?xml version="1.0"?>
                            <grammar version="1.0"
                            tag-format="first"
                             xmlns="http://www.w3.org/2001/06/grammar">
                              <rule id="2"> <one-of><item>"ff</item>
                              </one-of>
                              </rule>
                              <rule id="5>  <one-of><item>"ls</item>
                              </one-of>
                    
                              </grammar>
                """;
        messages.add("parsing error");
        files.add(file8);


        for (int i = 0; i < files.size(); i++) {

            testOneMessage(files.get(i), messages.get(i));
        }

    }

    private void testOneMessage(String file, String message) {

        boolean error = false;
        try {
            grammarService.addGrammarFromFile(file, true);
        } catch (ResponseStatusException e) {
            assert e.getReason().matches(message);
            error = true;
        }
        assert error;
    }

    private boolean predicate1(Rule rule) {
        boolean result = true;
        switch (rule.getLeftSymbol()) {
            case "<rule1>" -> result = rule.getRightSide().equals("123|smile <rule2>haha");
            case "<rule2>" -> result = rule.getRightSide().equals("cry");
            default -> result = false;
        }
        return result;
    }

}
